@@ -14,6 +14,7 @@ NULL
 #' ```
 #' p <- process$new(command = NULL, args,
 #'                  stdin = NULL, stdout = NULL, stderr = NULL,
+#'                  pty = FALSE, pty_options = list(),
 #'                  connections = list(), poll_connection = NULL,
 #'                  env = NULL, cleanup = TRUE, cleanup_tree = FALSE,
 #'                  wd = NULL, echo_cmd = FALSE, supervise = FALSE,
@@ -85,7 +86,7 @@ NULL
 #'     used as is, without a shell. They don't need to be escaped.
 #' * `stdin`: What to do with the standard input. Possible values:
 #'     `NULL`: set to the _null device_, i.e. no standard input is
-#'     provided; a string, supply the specified string as standard input;
+#'     provided; a file name, use this file as standard input;
 #'     `"|"`: create a (writeable) connection for stdin.
 #' * `stdout`: What to do with the standard output. Possible values:
 #'     `NULL`: discard it; a string, redirect it to this file;
@@ -95,6 +96,22 @@ NULL
 #'     `"|"`: create a connection for it; `"2>&1"`: redirect it to the
 #'     same connection (i.e. pipe or file) as `stdout`. `"2>&1"` is a
 #'     way to keep standard output and error correctly interleaved.
+#' * `pty`: Whether to create a pseudo terminal (pty) for the background
+#'     process. This is currently only supported on Unix systems, but
+#'     not supported on Solaris.
+#'     If it is `TRUE`, then the `stdin`, `stdout` and `stderr` arguments
+#'     must be `NULL`. If a pseudo terminal is created, then processx
+#'     will create pipes for standard input and standard output. There is
+#'     no separate pipe for standard error, because there is no way to
+#'     distinguish between stdout and stderr on a pty. Note that the
+#'     standard output connection of the pty is _blocking_, so we always
+#'     poll the standard output connection before reading from it using
+#'     the `$read_output()` method. Also, because `$read_output_lines()`
+#'     could still block if no complete line is available, this function
+#'     always fails if the process has a pty. Use `$read_output()` to
+#'     read from ptys.
+#' * `pty_options`: Unix pseudo terminal options, a named list. see
+#'     [default_pty_options()] for details and defaults.
 #' * `connections`: A list of connections to pass to the child process.
 #'     This is an experimental feature currently.
 #' * `poll_connection`: Whether to create an extra connection to the process
@@ -417,16 +434,16 @@ process <- R6::R6Class(
   public = list(
 
     initialize = function(command = NULL, args = character(),
-      stdin = NULL, stdout = NULL, stderr = NULL, connections = list(),
-      poll_connection = NULL, env = NULL, cleanup = TRUE,
-      cleanup_tree = FALSE, wd = NULL, echo_cmd = FALSE, supervise = FALSE,
-      windows_verbatim_args = FALSE, windows_hide_window = FALSE,
-      encoding = "",  post_process = NULL)
+      stdin = NULL, stdout = NULL, stderr = NULL, pty = FALSE,
+      pty_options = list(), connections = list(), poll_connection = NULL,
+      env = NULL, cleanup = TRUE, cleanup_tree = FALSE, wd = NULL,
+      echo_cmd = FALSE, supervise = FALSE, windows_verbatim_args = FALSE,
+      windows_hide_window = FALSE, encoding = "",  post_process = NULL)
 
       process_initialize(self, private, command, args, stdin,
-                         stdout, stderr, connections, poll_connection,
-                         env, cleanup, cleanup_tree, wd, echo_cmd,
-                         supervise, windows_verbatim_args,
+                         stdout, stderr, pty, pty_options, connections,
+                         poll_connection, env, cleanup, cleanup_tree, wd,
+                         echo_cmd, supervise, windows_verbatim_args,
                          windows_hide_window, encoding, post_process),
 
     finalize = function() {
@@ -590,6 +607,8 @@ process <- R6::R6Class(
     stdin = NULL,         # stdin argument or stream
     stdout = NULL,        # stdout argument or stream
     stderr = NULL,        # stderr argument or stream
+    pty = NULL,           # whether we should create a PTY
+    pty_options = NULL,   # various PTY options
     pstdin = NULL,        # the original stdin argument
     pstdout = NULL,       # the original stdout argument
     pstderr = NULL,       # the original stderr argument
@@ -633,23 +652,23 @@ process <- R6::R6Class(
 
 process_wait <- function(self, private, timeout) {
   "!DEBUG process_wait `private$get_short_name()`"
-  .Call(c_processx_wait, private$status, as.integer(timeout))
+  rethrow_call(c_processx_wait, private$status, as.integer(timeout))
   invisible(self)
 }
 
 process_is_alive <- function(self, private) {
   "!DEBUG process_is_alive `private$get_short_name()`"
-  .Call(c_processx_is_alive, private$status)
+  rethrow_call(c_processx_is_alive, private$status)
 }
 
 process_get_exit_status <- function(self, private) {
   "!DEBUG process_get_exit_status `private$get_short_name()`"
-  .Call(c_processx_get_exit_status, private$status)
+  rethrow_call(c_processx_get_exit_status, private$status)
 }
 
 process_signal <- function(self, private, signal) {
   "!DEBUG process_signal `private$get_short_name()` `signal`"
-  .Call(c_processx_signal, private$status, as.integer(signal))
+  rethrow_call(c_processx_signal, private$status, as.integer(signal))
 }
 
 process_interrupt <- function(self, private) {
@@ -659,13 +678,13 @@ process_interrupt <- function(self, private) {
     st <- run(get_tool("interrupt"), c(pid, "c"), error_on_status = FALSE)
     if (st$status == 0) TRUE else FALSE
   } else {
-    .Call(c_processx_interrupt, private$status)
+    rethrow_call(c_processx_interrupt, private$status)
   }
 }
 
 process_kill <- function(self, private, grace, close_connections) {
   "!DEBUG process_kill '`private$get_short_name()`', pid `self$get_pid()`"
-  ret <- .Call(c_processx_kill, private$status, as.numeric(grace))
+  ret <- rethrow_call(c_processx_kill, private$status, as.numeric(grace))
   if (close_connections) private$close_connections()
   ret
 }
@@ -673,9 +692,8 @@ process_kill <- function(self, private, grace, close_connections) {
 process_kill_tree <- function(self, private, grace, close_connections) {
   "!DEBUG process_kill_tree '`private$get_short_name()`', pid `self$get_pid()`"
   if (!ps::ps_is_supported()) {
-    stop(structure(
-      list(message = "kill_tree is not supported on this platform"),
-      class = c("not_implemented", "error", "condition")))
+    throw(new_not_implemented_error(
+      "kill_tree is not supported on this platform"))
   }
 
   ret <- get("ps_kill_tree", asNamespace("ps"))(private$tree_id)
@@ -688,7 +706,7 @@ process_get_start_time <- function(self, private) {
 }
 
 process_get_pid <- function(self, private) {
-  .Call(c_processx_get_pid, private$status)
+  rethrow_call(c_processx_get_pid, private$status)
 }
 
 process_is_supervised <- function(self, private) {
@@ -707,7 +725,7 @@ process_supervise <- function(self, private, status) {
 }
 
 process_get_result <- function(self, private) {
-  if (self$is_alive()) stop("Process is still alive")
+  if (self$is_alive()) throw(new_error("Process is still alive"))
   if (!private$post_process_done && is.function(private$post_process)) {
     private$post_process_result <- private$post_process()
     private$post_process_done <- TRUE
@@ -726,7 +744,21 @@ ps_method <- function(fun, self) {
 process_close_connections <- function(self, private) {
   for (f in c("stdin_pipe", "stdout_pipe", "stderr_pipe", "poll_pipe")) {
     if (!is.null(p <- private[[f]])) {
-      .Call(c_processx_connection_close, p)
+      rethrow_call(c_processx_connection_close, p)
     }
   }
 }
+
+#' Default options for pseudo terminals (ptys)
+#'
+#' @return Named list of default values of pty options.
+#'
+#' Options and default values:
+#' * `echo` whether to keep the echo on the terminal. `FALSE` turns echo
+#'   off.
+#'
+#' @export
+
+default_pty_options <- function() {
+  list(echo = FALSE)
+}
